rm(list=ls())
library(survival)
library(coda)
library(spBayesSurv)

folder.results <- 'C:/Users/fengd/Documents/papers/BDNNSurv/simulation/results/ddp/case1/'

n <- 5000
nvars <- 10

a=1.1; b=0.8
beta2=0.1

seeds <- 1:100
pickTimes <- c(0.07864246, 0.16315466, 0.26375659, 
               0.38902902, 0.55357536, 0.78438583, 
               1.13899336, 1.78226636, 3.44708184 )

for(ii in 1:100){
    set.seed(seeds[ii]+100)
    
    #simulate survival time
    u <- runif(n, 0, 1)
    z <- rbinom(n, 1, 0.5)
    nvars <- 10
    XX <- matrix(rnorm(nvars*n), nrow=n, ncol=nvars)
    rho <- 0.5    
    V <- matrix(c(1, rho, rho^2,
                  rho, 1, rho^2,
                  rho^2, rho, 1), 3, 3)
    qf <- rep(0, n)
    for(i in 1:n){
      qf[i] <- t(XX[i,1:3]) %*% V %*% XX[i,1:3]
    }    
    
    
    y1 <- as.vector(b*(exp(-log(u)*exp(-qf*beta2))-1)^(1/a)) #H(t)=log(1+(t/b)^a);b is the median (scale)
    print(quantile(y1, seq(0.1, 0.9, by=0.1)))
    
    #simulate censoing time
    last <- 15
    t.censor=0.2
    t <- rexp(n, t.censor)
    yc <- ifelse(t<last, t, last)
    time <- ifelse(y1<=yc, y1, yc)
    status <- ifelse(time<yc, 1, 0)        
    print(mean(status))
  
    #split data into training and test
    pick.train <- 1:(n*0.75)
    pick.test <- (n*0.75 + 1):n
    
    x_train  <- cbind(XX, z)[pick.train,]
    surv_train <- time[pick.train]
    cen_train <- status[pick.train]
    
    x_test  <- cbind(XX, z)[pick.test,]
    surv_test <- time[pick.test]
    cen_test <- status[pick.test]
    
 
    #run anovaddp
    nburn=500; nsave=1000; nskip=0;
    mcmc=list(nburn=nburn, nsave=nsave, nskip=nskip, ndisplay=100);
    prior = list(N=10, a0=2, b0=2);
    res1 = anovaDDP(formula = Surv(surv_train, cen_train)~x_train,  
                    prior=prior, mcmc=mcmc)
   
    #obtain survival probabilites and corresponding cis
    nw <- dim(res1$w)[1]
    niter <- dim(res1$w)[2]
    
    cdfPickTimesAll <- array(0, dim=c(nrow(x_test), niter, length(pickTimes)))
    for(ix in 1:nrow(x_test)){
        for(iter in 1:niter){
            cdfPickTimes <- rep(0, length(pickTimes))
            for(iw in 1:nw){
                cdfPickTimes <- cdfPickTimes + 
                    res1$w[iw,iter]*pnorm(log(pickTimes), 
                                          mean=c(1, x_test[ix,]) %*% res1$beta[,iw,iter], 
                                          sd=sqrt(res1$sigma2[iw,iter]))
            }
            cdfPickTimesAll[ix, iter, ] <- cdfPickTimes
        }
    }
        
    CDF <- apply(cdfPickTimesAll, c(2,3), mean)
    pred <- colMeans(CDF)
    pred.ci <- apply(CDF, 2, function(x) as.vector(HPDinterval(mcmc(x), 0.95)))
    print(pred.ci)
    
    save(pred, file=paste0(file=paste0(folder.results, 'ddp-point', ii, '.rda')))
    save(pred.ci, file=paste0(file=paste0(folder.results, 'ddp-ci', ii, '.rda')))
   }

#----------------------------------------------------------------------------------
rm(list=ls())
library(coda)
folder.results <- 'C:/Users/fengd/Documents/papers/BDNNSurv/simulation/results/ddp/case1/'
folder.summary <- "C:/Users/fengd/Documents/papers/BDNNSurv/simulation/summary/ddp/"

nTime <- 9
nRep <- 100
est.all <- matrix(0, nRep, nTime)
ci.all <- array(0, dim=c(nRep, nTime, 2))
for(ii in 1:nRep){
    load(file=paste0(file=paste0(folder.results, 'ddp-point', ii, '.rda')))
    load(file=paste0(file=paste0(folder.results, 'ddp-ci', ii, '.rda')))
    est.all[ii, ] <- pred
    ci.all[ii,,] <- t(pred.ci)
}


truth <- seq(0.1, 0.1*nTime, 0.1)
bias <- colMeans(est.all) - truth
sqrtmse <- sqrt(bias^2+ apply(est.all, 2, var))
cov <- rep(0, nTime)
for(i in 1:nTime){
    ci <- ci.all[,i,]
    cov[i] <- mean(ci[,1] < truth[i] & ci[,2]>truth[i])
}

bias; sqrtmse; cov