library(BART)
library(coda)

Runcode <- function(array_id=array_id){
    
    #-------------------------------------simulate data------------------------------------
    n <- 5000
    nvars <- 10
    
    a=1.1; b=0.8
    beta2=0.1
    
    set.seed(array_id + 100)
    
    #simulate survival time
    u <- runif(n, 0, 1)
    z <- rbinom(n, 1, 0.5)
    nvars <- 10
    XX <- matrix(rnorm(nvars*n), nrow=n, ncol=nvars)
    rho <- 0.5    
    V <- matrix(c(1, rho, rho^2,
                  rho, 1, rho^2,
                  rho^2, rho, 1), 3, 3)
    qf <- rep(0, n)
    for(i in 1:n){
        qf[i] <- t(XX[i,1:3]) %*% V %*% XX[i,1:3]
    }    
    
    
    y1 <- as.vector(b*(exp(-log(u)*exp(-qf*beta2))-1)^(1/a)) 

    #simulate censoing time
    last <- 15
    t.censor <- 0.2
    t <- rexp(n, t.censor)
    yc <- ifelse(t<last, t, last)
    time <- ifelse(y1<=yc, y1, yc)
    status <- ifelse(time<yc, 1, 0)        

    #split data into training and test
    pick.train <- 1:(n*0.75)
    pick.test <- (n*0.75 + 1):n
    
    x_train  <- cbind(XX, z)[pick.train,]
    surv_train <- time[pick.train]
    cen_train <- status[pick.train]
    
    x_test  <- cbind(XX, z)[pick.test,]
    surv_test <- time[pick.test]
    cen_test <- status[pick.test]
   
    #data normalization
    mean <- apply(x_train, 2, mean)
    std <- apply(x_train, 2, sd)
    x_train <- scale(x_train, center = mean, scale = std)
    x_test <- scale(x_test, center = mean, scale = std)
    
    #run BART
    time <- c(0.07864246, 0.16315466, 0.26375659, 0.38902902, 0.55357536, 0.78438583, 1.13899336, 1.78226636, 3.44708184)
    
    pick <- pd.mu <- pd.025 <- pd.975 <- rep(0, length(time))
    
    post <- surv.bart(x.train=x_train, times=surv_train, delta=cen_train, K=200, x.test=x_test,
                      ndpost=1000, nskip=250, keepevery = 1L)
    
    #obtain survival probabilites and corresponding cis
    K <- length(post$times)
    N <- nrow(x_test)
    M <- nrow(post$yhat.test)
  
    for(j in 1:length(time)){
        pick[j] <- which.min(abs(post$times - time[j]))
    }
    
    for(j in 1:length(time)) {
        h <- seq(pick[j], N*K, by=K)
        pd <- 1 - apply(post$surv.test[ , h], 1, mean)
        pd.mu[j] <- mean(pd)
        pd.025[j] <- quantile(pd, 0.025)
        pd.975[j] <- quantile(pd, 0.975)
    }
    
    rm(post)
    
    res <- rbind(pd.mu, pd.025, pd.975)
    
    return(res)
}

array_id <- as.numeric(Sys.getenv('SLURM_ARRAY_TASK_ID'));

res=Runcode(array_id)

write.csv(res, paste("/home/zhaolili/Feng/Results/bart-",  array_id,".csv",sep=""),row.names = FALSE);


#--------------------------The code to summarize the results--------------------
#Let dd be the 300*9 matrix obtained 
truth <- seq(0.1, 0.9, by=0.1)
est <- dd[seq(1, 300, by=3),]
ci.lb <- dd[seq(2, 300, by=3),]
ci.ub <- dd[seq(3, 300, by=3),]
bias <- mse <- cov <- rep(0, length(truth))
for(i in 1:length(truth)){
    bias[i] <- mean(est[,i]) - truth[i]
    mse[i] <- bias[i]^2 + var(est[,i])
    cov[i] <- mean(ci.lb[,i] < truth[i] & ci.ub[,i] > truth[i])
    
}
bias; sqrt(mse); cov

