
#==================================Obtain true values by simulation===========================================
a=1.1; b=0.8
beta2=0.1

n <- 10000000
set.seed(100)
u <- runif(n, 0, 1)
z <- rbinom(n, 1, 0.5)
nvars <- 10
XX <- matrix(rnorm(nvars*n), nrow=n, ncol=nvars)
rho <- 0.5    
V <- matrix(c(1, rho, rho^2,
              rho, 1, rho^2,
              rho^2, rho, 1), 3, 3)
qf <- rep(0, n)
for(i in 1:n){
    qf[i] <- t(XX[i,1:3]) %*% V %*% XX[i,1:3]
}    


y1 <- as.vector(b*(exp(-log(u)*exp(-qf*beta2))-1)^(1/a)) 
print(quantile(y1, seq(0.1, 0.9, by=0.1)))
#       10%        20%        30%        40%        50%        60%        70%        80%        90% 
#0.07864246 0.16315466 0.26375659 0.38902902 0.55357536 0.78438583 1.13899336 1.78226636 3.44708184 

#=========================================simulate data=======================================================

rm(list=ls())
library(survival)
library(pseudo)

#--------------------------------------------------------------------------
#obtain pseudo-values for a data set
getPseudo <- function(time, event, tmax){
    
    pseudodt <- 1 - pseudosurv(time=time, event=event,
                               tmax=tmax)$pseudo
    
    pseudodt
}

#--------------------------------------------------------------------------
folder <- 'C:/Users/fengd/Documents/papers/BDNNSurv/simulation/data/case1/'

n <- 5000
nvars <- 10
a=1.1; b=0.8
beta2=0.1

seeds <- rep(1:100, 9)
pickTimes <- rep(c(0.07864246, 0.16315466, 0.26375659, 0.38902902, 0.55357536, 0.78438583, 
                   1.13899336, 1.78226636, 3.44708184 ), each=100)

for(ii in 1:900){
    set.seed(seeds[ii]+100)

    #simulate survival time
    u <- runif(n, 0, 1)
    z <- rbinom(n, 1, 0.5)
    nvars <- 10
    XX <- matrix(rnorm(nvars*n), nrow=n, ncol=nvars)
    rho <- 0.5    
    V <- matrix(c(1, rho, rho^2,
                  rho, 1, rho^2,
                  rho^2, rho, 1), 3, 3)
    qf <- rep(0, n)
    for(i in 1:n){
      qf[i] <- t(XX[i,1:3]) %*% V %*% XX[i,1:3]
    }    
    
    
    y1 <- as.vector(b*(exp(-log(u)*exp(-qf*beta2))-1)^(1/a)) 
   
    #simulate censoing time
    last <- 15
    t.censor=0.2
    t <- rexp(n, t.censor)
    yc <- ifelse(t<last, t, last)
    time <- ifelse(y1<=yc, y1, yc)
    status <- ifelse(time<yc, 1, 0)        

    #split data into training and test
    pick.train <- 1:(n*0.75)
    pick.test <- (n*0.75 + 1):n
    
    x_train  <- cbind(XX, z)[pick.train,]
    surv_train <- time[pick.train]
    cen_train <- status[pick.train]
    
    x_test  <- cbind(XX, z)[pick.test,]
    surv_test <- time[pick.test]
    cen_test <- status[pick.test]
    
    #data normalization
    mean <- apply(x_train, 2, mean)
    std <- apply(x_train, 2, sd)
    x_train <- scale(x_train, center = mean, scale = std)
    x_test <- scale(x_test, center = mean, scale = std)
    
    #calculate pseudo values
    x_train.all <- x_train
    x_test.all <- x_test
   
    pickTime <- pickTimes[ii]
    pseudo <- getPseudo(surv_train, cen_train, pickTime)
    y_train.all <- as.vector(pseudo)
    
    write.table(x_train.all, file=paste0(folder, 'sim-xtrainall-', ii, '.csv'), sep = ',', row.names = FALSE, col.names = FALSE)
    write.table(y_train.all, file=paste0(folder, 'sim-ytrainall-', ii, '.csv'), sep = ',', row.names = FALSE, col.names = FALSE)
    write.table(x_test.all, file=paste0(folder, 'sim-xtestall-', ii, '.csv'), sep = ',', row.names = FALSE, col.names = FALSE)
    
}

#======================================Summarize results from BDNNSruv==============================================

#Run this code after running bdnnsurv-initial.py and bdnnsurv.py to summarize results

rm(list=ls())
library(coda)
folder.results <- "C:/Users/fengd/Documents/papers/BDNNSurv/simulation/results/case1/"
folder.summary <- "C:/Users/fengd/Documents/papers/BDNNSurv/simulation/summary/bdnnsurv/"

nTime <- 9
nRep <- 100
est.all <- matrix(0, nRep, nTime)
ci.all <- array(0, dim=c(nRep, nTime, 2))
for(jj in 1:nTime)
    for(ii in 1:nRep){
        pred <- read.csv(paste0(folder.results, 'sim-ytestall-', (jj-1)*nRep+ii, ".csv"), header=FALSE)
        post <- rowMeans(pred)
        est.all[ii, jj] <- mean(post)
        ci.all[ii,jj,1:2] <- as.vector(HPDinterval(mcmc(post), 0.95))
    }

save(est.all, file=paste0(folder.summary, 'case1-est.rda'))
save(ci.all, file=paste0(folder.summary, 'case1-ci.rda'))

#-------------------------------------------------------------------------------------------
rm(list=ls())
folder.summary <- "C:/Users/fengd/Documents/papers/BDNNSurv/simulation/summary/bdnnsurv/"

load(file=paste0(folder.summary, 'case1-est.rda'))
load(file=paste0(folder.summary, 'case1-ci.rda'))


truth <- seq(0.1, 0.9, 0.1)
bias <- colMeans(est.all) - truth
sqrtmse <- sqrt(bias^2+ apply(est.all, 2, var))
cov <- rep(0, 9)
for(i in 1:9){
    ci <- ci.all[,i,]
    cov[i] <- mean(ci[,1] < truth[i] & ci[,2]>truth[i])
}


bias; sqrtmse; cov