import theano
import pymc3 as pm
import theano.tensor as T
import numpy as np 
import pickle

fin = "C:/Users/fengd/Documents/papers/BDNNSurv/simulation/data/case1/"
fout = "C:/Users/fengd/Documents/papers/BDNNSurv/simulation/results/case1/"

#---------------------------------------------------------------------
def construct_nn(ann_input, ann_output,
                 init_in_1, init_1_2, init_2_out,
                 initb_in_1, initb_1_2, initb_2_out):
    numFeatures = ann_input.get_value().shape[1]
      
    with pm.Model() as neural_network:
        weights_in_1 = pm.Normal('w_in_1', 0, sd=1,
                                 shape=(numFeatures, 8), testval=init_in_1)
        weights_1_2 = pm.Normal('w_1_2', 0, sd=1,
                                shape=(8, 4), testval=init_1_2)
        weights_2_out = pm.Normal('w_2_out', 0, sd=1,
                                  shape=(4, ), testval=init_2_out)
        bias_in_1 = pm.Normal('b_in_1', 0, sd=1, shape=(8,), testval=initb_in_1)
        bias_1_2 = pm.Normal('b_1_2', 0, sd=1, shape=(4,), testval=initb_1_2)
        bias_2_out = pm.Normal('b_4_out', 0, sd=1, shape=(1,), testval=initb_2_out)
   
        act_1 = pm.math.tanh(pm.math.dot(ann_input, weights_in_1) + bias_in_1)
        act_2 = pm.math.tanh(pm.math.dot(act_1, weights_1_2) + bias_1_2)
        act_out = pm.math.sigmoid(pm.math.dot(act_2, weights_2_out) + bias_2_out)
        sigma = pm.Uniform('sigma', lower=0, upper=0.2)
        out = pm.Normal('out', mu=act_out, sd=sigma, observed=ann_output)
    return neural_network

#------------------------------------------------------------------------------------------
for i in range(1,901):
    x_train = np.loadtxt(fname=fin + "sim-xtrainall-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    y_train = np.loadtxt(fname=fin + "sim-ytrainall-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    N = x_train.shape[0]
    y_train = np.reshape(y_train, (1, N))
    #--------------------------------------------------------
    init_in_1 = np.loadtxt(fname=fout + "sim-keras-W0-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    init_1_2 = np.loadtxt(fname=fout + "sim-keras-W1-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    init_2_out = np.loadtxt(fname=fout + "sim-keras-W2-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    initb_in_1 = np.loadtxt(fname=fout + "sim-keras-b0-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    initb_1_2 = np.loadtxt(fname=fout + "sim-keras-b1-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    initb_2_out = np.loadtxt(fname=fout + "sim-keras-b2-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    #----------------------------------------------------------
    ann_input = theano.shared(x_train)
    ann_output = theano.shared(y_train)
    neural_network = construct_nn(ann_input, ann_output, init_in_1, init_1_2, init_2_out,
                                 initb_in_1, initb_1_2, initb_2_out)
    #---------------------------------------------------------------------
    with neural_network:
        inference = pm.ADVI()
        approx = pm.fit(n=100000, method=inference)
    #--------------------------------------------------------------------
    trace = approx.sample(draws=1000)
    preds = pm.sample_ppc(trace, model=neural_network, samples=1000) 
    y_preds = preds['out']
    np.corrcoef(y_preds.mean(axis=0), y_train)
    #--------------------------------------------------------------------
    x_test = np.loadtxt(fname=fin + "sim-xtestall-" + str(i) + ".csv",delimiter=",",skiprows=0).astype(np.float32)
    ann_input.set_value(x_test)
    with neural_network:    
        ann_input.set_value(x_test)
        ppc = pm.sample_ppc(trace, 1000)
        pred = ppc['out']
    np.savetxt(fout + "sim-ytestall-" + str(i) + ".csv", pred, delimiter=",")




